@@ -143,6 +143,7 @@ enum class OperatingSystem : uint8_t {
   OSX10_13,
   OSX10_14,
   OSX10_15,
+  OSX11_0,
   Android,
   Ios
 };
@@ -170,6 +171,8 @@ enum class DeviceFamily : uint8_t {
   MicrosoftAll,
   ParallelsAll,
   QualcommAll,
+  AppleAll,
+  AmazonAll,
   IntelGMA500,
   IntelGMA900,
   IntelGMA950,
@@ -178,8 +181,9 @@ enum class DeviceFamily : uint8_t {
   IntelGMAX4500HD,
   IntelHDGraphicsToIvyBridge,
   IntelHDGraphicsToSandyBridge,
-  IntelHDGraphicsToHaswell,
-  IntelHD3000,
+  IntelHaswell,
+  IntelSandyBridge,
+  IntelGen7Baytrail,
   IntelHD520,
   IntelMobileHDGraphics,
   NvidiaBlockD3D9Layers,
@@ -193,7 +197,7 @@ enum class DeviceFamily : uint8_t {
   Bug1155608,
   Bug1207665,
   Bug1447141,
-  NvidiaBlockWebRender,
+  AmdR600,
   NvidiaRolloutWebRender,
   IntelRolloutWebRender,
   IntelModernRolloutWebRender,
@@ -209,7 +213,13 @@ enum class DeviceVendor : uint8_t {
   ATI,
   Microsoft,
   Parallels,
+  VMWare,
+  VirtualBox,
   Qualcomm,
+  MicrosoftBasic,
+  MicrosoftHyperV,
+  Apple,
+  Amazon,
 
   Max
 };
@@ -224,10 +234,18 @@ enum DriverVendor : uint8_t {
   MesaLLVMPipe,
   MesaSoftPipe,
   MesaSWRast,
+  // AMD
+  MesaR600,
+  // Nouveau: Open-source nvidia
+  MesaNouveau,
   // A generic ID to be provided when we can't determine the DRI driver on Mesa.
   MesaUnknown,
   // Wildcard for all non-Mesa drivers.
   NonMesaAll,
+  // Wildcard for all hardware Mesa drivers.
+  HardwareMesaAll,
+  // Wildcard for all software Mesa drivers.
+  SoftwareMesaAll,
 
   Max
 };
@@ -247,6 +265,8 @@ enum class DesktopEnvironment : uint8_t {
   Pantheon,
   LXQT,
   Deepin,
+  Dwm,
+  Budgie,
   Unknown,
   Max
 };
@@ -254,10 +274,13 @@ enum class DesktopEnvironment : uint8_t {
 enum class WindowProtocol : uint8_t {
   All,  // There is an assumption that this is the first enum
   X11,
+  XWayland,
   Wayland,
   WaylandDRM,
-  // Wildcard for all Wayland variants.
+  // Wildcard for all Wayland variants, excluding XWayland.
   WaylandAll,
+  // Wildcard for all X11 variants, including XWayland.
+  X11All,
   Max
 };
 
@@ -456,7 +479,7 @@ inline bool ParseDriverVersion(const nsAString& aVersion,
                                uint64_t* aNumericVersion) {
   *aNumericVersion = 0;
 
-#if defined(XP_WIN) || defined(MOZ_X11)
+#if defined(XP_WIN) || defined(MOZ_WIDGET_GTK)
   int a, b, c, d;
   char aStr[8], bStr[8], cStr[8], dStr[8];
   /* honestly, why do I even bother */
