@@ -19,14 +19,15 @@
 #include "X11UndefineNone.h"
 #include "IMContextWrapper.h"
 #include "WidgetUtils.h"
+#include "WidgetUtilsGtk.h"
 #include "keysym2ucs.h"
 #include "nsContentUtils.h"
 #include "nsGtkUtils.h"
 #include "nsIBidiKeyboard.h"
 #include "nsPrintfCString.h"
+#include "nsReadableUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsWindow.h"
-#include "gfxPlatformGtk.h"
 
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/MouseEvents.h"
@@ -74,13 +75,13 @@ static const char* GetStatusName(nsEventStatus aStatus) {
 static const nsCString GetKeyLocationName(uint32_t aLocation) {
   switch (aLocation) {
     case eKeyLocationLeft:
-      return NS_LITERAL_CSTRING("KEY_LOCATION_LEFT");
+      return "KEY_LOCATION_LEFT"_ns;
     case eKeyLocationRight:
-      return NS_LITERAL_CSTRING("KEY_LOCATION_RIGHT");
+      return "KEY_LOCATION_RIGHT"_ns;
     case eKeyLocationStandard:
-      return NS_LITERAL_CSTRING("KEY_LOCATION_STANDARD");
+      return "KEY_LOCATION_STANDARD"_ns;
     case eKeyLocationNumpad:
-      return NS_LITERAL_CSTRING("KEY_LOCATION_NUMPAD");
+      return "KEY_LOCATION_NUMPAD"_ns;
     default:
       return nsPrintfCString("Unknown (0x%04X)", aLocation);
   }
@@ -89,87 +90,87 @@ static const nsCString GetKeyLocationName(uint32_t aLocation) {
 static const nsCString GetCharacterCodeName(char16_t aChar) {
   switch (aChar) {
     case 0x0000:
-      return NS_LITERAL_CSTRING("NULL (0x0000)");
+      return "NULL (0x0000)"_ns;
     case 0x0008:
-      return NS_LITERAL_CSTRING("BACKSPACE (0x0008)");
+      return "BACKSPACE (0x0008)"_ns;
     case 0x0009:
-      return NS_LITERAL_CSTRING("CHARACTER TABULATION (0x0009)");
+      return "CHARACTER TABULATION (0x0009)"_ns;
     case 0x000A:
-      return NS_LITERAL_CSTRING("LINE FEED (0x000A)");
+      return "LINE FEED (0x000A)"_ns;
     case 0x000B:
-      return NS_LITERAL_CSTRING("LINE TABULATION (0x000B)");
+      return "LINE TABULATION (0x000B)"_ns;
     case 0x000C:
-      return NS_LITERAL_CSTRING("FORM FEED (0x000C)");
+      return "FORM FEED (0x000C)"_ns;
     case 0x000D:
-      return NS_LITERAL_CSTRING("CARRIAGE RETURN (0x000D)");
+      return "CARRIAGE RETURN (0x000D)"_ns;
     case 0x0018:
-      return NS_LITERAL_CSTRING("CANCEL (0x0018)");
+      return "CANCEL (0x0018)"_ns;
     case 0x001B:
-      return NS_LITERAL_CSTRING("ESCAPE (0x001B)");
+      return "ESCAPE (0x001B)"_ns;
     case 0x0020:
-      return NS_LITERAL_CSTRING("SPACE (0x0020)");
+      return "SPACE (0x0020)"_ns;
     case 0x007F:
-      return NS_LITERAL_CSTRING("DELETE (0x007F)");
+      return "DELETE (0x007F)"_ns;
     case 0x00A0:
-      return NS_LITERAL_CSTRING("NO-BREAK SPACE (0x00A0)");
+      return "NO-BREAK SPACE (0x00A0)"_ns;
     case 0x00AD:
-      return NS_LITERAL_CSTRING("SOFT HYPHEN (0x00AD)");
+      return "SOFT HYPHEN (0x00AD)"_ns;
     case 0x2000:
-      return NS_LITERAL_CSTRING("EN QUAD (0x2000)");
+      return "EN QUAD (0x2000)"_ns;
     case 0x2001:
-      return NS_LITERAL_CSTRING("EM QUAD (0x2001)");
+      return "EM QUAD (0x2001)"_ns;
     case 0x2002:
-      return NS_LITERAL_CSTRING("EN SPACE (0x2002)");
+      return "EN SPACE (0x2002)"_ns;
     case 0x2003:
-      return NS_LITERAL_CSTRING("EM SPACE (0x2003)");
+      return "EM SPACE (0x2003)"_ns;
     case 0x2004:
-      return NS_LITERAL_CSTRING("THREE-PER-EM SPACE (0x2004)");
+      return "THREE-PER-EM SPACE (0x2004)"_ns;
     case 0x2005:
-      return NS_LITERAL_CSTRING("FOUR-PER-EM SPACE (0x2005)");
+      return "FOUR-PER-EM SPACE (0x2005)"_ns;
     case 0x2006:
-      return NS_LITERAL_CSTRING("SIX-PER-EM SPACE (0x2006)");
+      return "SIX-PER-EM SPACE (0x2006)"_ns;
     case 0x2007:
-      return NS_LITERAL_CSTRING("FIGURE SPACE (0x2007)");
+      return "FIGURE SPACE (0x2007)"_ns;
     case 0x2008:
-      return NS_LITERAL_CSTRING("PUNCTUATION SPACE (0x2008)");
+      return "PUNCTUATION SPACE (0x2008)"_ns;
     case 0x2009:
-      return NS_LITERAL_CSTRING("THIN SPACE (0x2009)");
+      return "THIN SPACE (0x2009)"_ns;
     case 0x200A:
-      return NS_LITERAL_CSTRING("HAIR SPACE (0x200A)");
+      return "HAIR SPACE (0x200A)"_ns;
     case 0x200B:
-      return NS_LITERAL_CSTRING("ZERO WIDTH SPACE (0x200B)");
+      return "ZERO WIDTH SPACE (0x200B)"_ns;
     case 0x200C:
-      return NS_LITERAL_CSTRING("ZERO WIDTH NON-JOINER (0x200C)");
+      return "ZERO WIDTH NON-JOINER (0x200C)"_ns;
     case 0x200D:
-      return NS_LITERAL_CSTRING("ZERO WIDTH JOINER (0x200D)");
+      return "ZERO WIDTH JOINER (0x200D)"_ns;
     case 0x200E:
-      return NS_LITERAL_CSTRING("LEFT-TO-RIGHT MARK (0x200E)");
+      return "LEFT-TO-RIGHT MARK (0x200E)"_ns;
     case 0x200F:
-      return NS_LITERAL_CSTRING("RIGHT-TO-LEFT MARK (0x200F)");
+      return "RIGHT-TO-LEFT MARK (0x200F)"_ns;
     case 0x2029:
-      return NS_LITERAL_CSTRING("PARAGRAPH SEPARATOR (0x2029)");
+      return "PARAGRAPH SEPARATOR (0x2029)"_ns;
     case 0x202A:
-      return NS_LITERAL_CSTRING("LEFT-TO-RIGHT EMBEDDING (0x202A)");
+      return "LEFT-TO-RIGHT EMBEDDING (0x202A)"_ns;
     case 0x202B:
-      return NS_LITERAL_CSTRING("RIGHT-TO-LEFT EMBEDDING (0x202B)");
+      return "RIGHT-TO-LEFT EMBEDDING (0x202B)"_ns;
     case 0x202D:
-      return NS_LITERAL_CSTRING("LEFT-TO-RIGHT OVERRIDE (0x202D)");
+      return "LEFT-TO-RIGHT OVERRIDE (0x202D)"_ns;
     case 0x202E:
-      return NS_LITERAL_CSTRING("RIGHT-TO-LEFT OVERRIDE (0x202E)");
+      return "RIGHT-TO-LEFT OVERRIDE (0x202E)"_ns;
     case 0x202F:
-      return NS_LITERAL_CSTRING("NARROW NO-BREAK SPACE (0x202F)");
+      return "NARROW NO-BREAK SPACE (0x202F)"_ns;
     case 0x205F:
-      return NS_LITERAL_CSTRING("MEDIUM MATHEMATICAL SPACE (0x205F)");
+      return "MEDIUM MATHEMATICAL SPACE (0x205F)"_ns;
     case 0x2060:
-      return NS_LITERAL_CSTRING("WORD JOINER (0x2060)");
+      return "WORD JOINER (0x2060)"_ns;
     case 0x2066:
-      return NS_LITERAL_CSTRING("LEFT-TO-RIGHT ISOLATE (0x2066)");
+      return "LEFT-TO-RIGHT ISOLATE (0x2066)"_ns;
     case 0x2067:
-      return NS_LITERAL_CSTRING("RIGHT-TO-LEFT ISOLATE (0x2067)");
+      return "RIGHT-TO-LEFT ISOLATE (0x2067)"_ns;
     case 0x3000:
-      return NS_LITERAL_CSTRING("IDEOGRAPHIC SPACE (0x3000)");
+      return "IDEOGRAPHIC SPACE (0x3000)"_ns;
     case 0xFEFF:
-      return NS_LITERAL_CSTRING("ZERO WIDTH NO-BREAK SPACE (0xFEFF)");
+      return "ZERO WIDTH NO-BREAK SPACE (0xFEFF)"_ns;
     default: {
       if (aChar < ' ' || (aChar >= 0x80 && aChar < 0xA0)) {
         return nsPrintfCString("control (0x%04X)", aChar);
@@ -190,17 +191,14 @@ static const nsCString GetCharacterCodeName(char16_t aChar) {
 static const nsCString GetCharacterCodeNames(const char16_t* aChars,
                                              uint32_t aLength) {
   if (!aLength) {
-    return NS_LITERAL_CSTRING("\"\"");
+    return "\"\""_ns;
   }
   nsCString result;
-  for (uint32_t i = 0; i < aLength; ++i) {
-    if (!result.IsEmpty()) {
-      result.AppendLiteral(", ");
-    } else {
-      result.AssignLiteral("\"");
-    }
-    result.Append(GetCharacterCodeName(aChars[i]));
-  }
+  result.AssignLiteral("\"");
+  StringJoinAppend(result, ", "_ns, Span{aChars, aLength},
+                   [](nsACString& dest, const char16_t charValue) {
+                     dest.Append(GetCharacterCodeName(charValue));
+                   });
   result.AppendLiteral("\"");
   return result;
 }
@@ -348,7 +346,7 @@ KeymapWrapper::KeymapWrapper()
 
   g_object_ref(mGdkKeymap);
 
-  if (gfxPlatformGtk::GetPlatform()->IsX11Display()) {
+  if (GdkIsX11Display()) {
     InitXKBExtension();
   }
 
@@ -367,7 +365,7 @@ void KeymapWrapper::Init() {
   mModifierKeys.Clear();
   memset(mModifierMasks, 0, sizeof(mModifierMasks));
 
-  if (gfxPlatformGtk::GetPlatform()->IsX11Display()) {
+  if (GdkIsX11Display()) {
     InitBySystemSettingsX11();
   }
 #ifdef MOZ_WAYLAND
@@ -757,8 +755,8 @@ static void gdk_registry_handle_global(void* data, struct wl_registry* registry,
                                        uint32_t id, const char* interface,
                                        uint32_t version) {
   if (strcmp(interface, "wl_seat") == 0) {
-    wl_seat* seat =
-        (wl_seat*)wl_registry_bind(registry, id, &wl_seat_interface, 1);
+    auto* seat =
+        WaylandRegistryBind<wl_seat>(registry, id, &wl_seat_interface, 1);
     wl_seat_add_listener(seat, &seat_listener, data);
   }
 }
@@ -1028,6 +1026,41 @@ uint32_t KeymapWrapper::ComputeKeyModifiers(guint aModifierState) {
   return keyModifiers;
 }
 
+/* static */
+guint KeymapWrapper::ConvertWidgetModifierToGdkState(
+    nsIWidget::Modifiers aNativeModifiers) {
+  if (!aNativeModifiers) {
+    return 0;
+  }
+  struct ModifierMapEntry {
+    nsIWidget::Modifiers mWidgetModifier;
+    Modifier mModifier;
+  };
+  // TODO: Currently, we don't treat L/R of each modifier on Linux.
+  // TODO: No proper native modifier for Level5.
+  static constexpr ModifierMapEntry sModifierMap[] = {
+      {nsIWidget::CAPS_LOCK, Modifier::CAPS_LOCK},
+      {nsIWidget::NUM_LOCK, Modifier::NUM_LOCK},
+      {nsIWidget::SHIFT_L, Modifier::SHIFT},
+      {nsIWidget::SHIFT_R, Modifier::SHIFT},
+      {nsIWidget::CTRL_L, Modifier::CTRL},
+      {nsIWidget::CTRL_R, Modifier::CTRL},
+      {nsIWidget::ALT_L, Modifier::ALT},
+      {nsIWidget::ALT_R, Modifier::ALT},
+      {nsIWidget::ALTGRAPH, Modifier::LEVEL3},
+      {nsIWidget::COMMAND_L, Modifier::SUPER},
+      {nsIWidget::COMMAND_R, Modifier::SUPER}};
+
+  guint state = 0;
+  KeymapWrapper* instance = GetInstance();
+  for (const ModifierMapEntry& entry : sModifierMap) {
+    if (aNativeModifiers & entry.mWidgetModifier) {
+      state |= instance->GetModifierMask(entry.mModifier);
+    }
+  }
+  return state;
+}
+
 /* static */
 void KeymapWrapper::InitInputEvent(WidgetInputEvent& aInputEvent,
                                    guint aModifierState) {
@@ -1072,26 +1105,27 @@ void KeymapWrapper::InitInputEvent(WidgetInputEvent& aInputEvent,
   WidgetMouseEventBase& mouseEvent = *aInputEvent.AsMouseEventBase();
   mouseEvent.mButtons = 0;
   if (aModifierState & GDK_BUTTON1_MASK) {
-    mouseEvent.mButtons |= MouseButtonsFlag::eLeftFlag;
+    mouseEvent.mButtons |= MouseButtonsFlag::ePrimaryFlag;
   }
   if (aModifierState & GDK_BUTTON3_MASK) {
-    mouseEvent.mButtons |= MouseButtonsFlag::eRightFlag;
+    mouseEvent.mButtons |= MouseButtonsFlag::eSecondaryFlag;
   }
   if (aModifierState & GDK_BUTTON2_MASK) {
     mouseEvent.mButtons |= MouseButtonsFlag::eMiddleFlag;
   }
 
   if (doLog) {
-    MOZ_LOG(gKeymapWrapperLog, LogLevel::Debug,
-            ("%p InitInputEvent, aInputEvent has mButtons, "
-             "aInputEvent.mButtons=0x%04X (Left: %s, Right: %s, Middle: %s, "
-             "4th (BACK): %s, 5th (FORWARD): %s)",
-             keymapWrapper, mouseEvent.mButtons,
-             GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::eLeftFlag),
-             GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::eRightFlag),
-             GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::eMiddleFlag),
-             GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::e4thFlag),
-             GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::e5thFlag)));
+    MOZ_LOG(
+        gKeymapWrapperLog, LogLevel::Debug,
+        ("%p InitInputEvent, aInputEvent has mButtons, "
+         "aInputEvent.mButtons=0x%04X (Left: %s, Right: %s, Middle: %s, "
+         "4th (BACK): %s, 5th (FORWARD): %s)",
+         keymapWrapper, mouseEvent.mButtons,
+         GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::ePrimaryFlag),
+         GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::eSecondaryFlag),
+         GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::eMiddleFlag),
+         GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::e4thFlag),
+         GetBoolName(mouseEvent.mButtons & MouseButtonsFlag::e5thFlag)));
   }
 }
 
@@ -1531,6 +1565,7 @@ void KeymapWrapper::HandleKeyPressEvent(nsWindow* aWindow,
               ("  HandleKeyPressEvent(), dispatched \"Forward\" command "
                "event"));
       return;
+    case GDK_Reload:
     case GDK_Refresh:
       aWindow->DispatchCommandEvent(nsGkAtoms::Reload);
       return;
@@ -1726,7 +1761,7 @@ void KeymapWrapper::InitKeyEvent(WidgetKeyboardEvent& aKeyEvent,
   // key release events, the result isn't what we want.
   guint modifierState = aGdkKeyEvent->state;
   GdkDisplay* gdkDisplay = gdk_display_get_default();
-  if (aGdkKeyEvent->is_modifier && GDK_IS_X11_DISPLAY(gdkDisplay)) {
+  if (aGdkKeyEvent->is_modifier && GdkIsX11Display(gdkDisplay)) {
     Display* display = gdk_x11_display_get_xdisplay(gdkDisplay);
     if (XEventsQueued(display, QueuedAfterReading)) {
       XEvent nextEvent;
